import vonage
from util import *

import sys

bytes_type = bytes

@responses.activate
def test_send_ussd_push_message(client, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/ussd/json")

    params = {"from": "MyCompany20", "to": "447525856424", "text": "Hello"}

    assert isinstance(client.send_ussd_push_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "from=MyCompany20" in request_body()
    assert "to=447525856424" in request_body()
    assert "text=Hello" in request_body()


@responses.activate
def test_send_ussd_prompt_message(client, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/ussd-prompt/json")

    params = {"from": "long-virtual-number", "to": "447525856424", "text": "Hello"}

    assert isinstance(client.send_ussd_prompt_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "from=long-virtual-number" in request_body()
    assert "to=447525856424" in request_body()
    assert "text=Hello" in request_body()


@responses.activate
def test_send_2fa_message(client, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/sc/us/2fa/json")

    params = {"to": "16365553226", "pin": "1234"}

    assert isinstance(client.send_2fa_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "to=16365553226" in request_body()
    assert "pin=1234" in request_body()


@responses.activate
def test_send_event_alert_message(client, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/sc/us/alert/json")

    params = {"to": "16365553226", "server": "host", "link": "http://example.com/"}

    assert isinstance(client.send_event_alert_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "to=16365553226" in request_body()
    assert "server=host" in request_body()
    assert "link=http%3A%2F%2Fexample.com%2F" in request_body()


@responses.activate
def test_send_marketing_message(client, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/sc/us/marketing/json")

    params = {
        "from": "short-code",
        "to": "16365553226",
        "keyword": "NEXMO",
        "text": "Hello",
    }

    assert isinstance(client.send_marketing_message(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "from=short-code" in request_body()
    assert "to=16365553226" in request_body()
    assert "keyword=NEXMO" in request_body()
    assert "text=Hello" in request_body()


@responses.activate
def test_get_event_alert_numbers(client, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/sc/us/alert/opt-in/query/json")

    assert isinstance(client.get_event_alert_numbers(), dict)
    assert request_user_agent() == dummy_data.user_agent


@responses.activate
def test_resubscribe_event_alert_number(client, dummy_data):
    stub(responses.POST, "https://rest.nexmo.com/sc/us/alert/opt-in/manage/json")

    params = {"msisdn": "441632960960"}

    assert isinstance(client.resubscribe_event_alert_number(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "msisdn=441632960960" in request_body()


def test_check_signature(dummy_data):
    params = {
        "a": "1",
        "b": "2",
        "timestamp": "1461605396",
        "sig": "6af838ef94998832dbfc29020b564830",
    }

    client = vonage.Client(
        key=dummy_data.api_key, secret=dummy_data.api_secret, signature_secret="secret"
    )

    assert client.check_signature(params)


def test_signature(client, dummy_data):
    params = {"a": "1", "b": "2", "timestamp": "1461605396"}
    client = vonage.Client(
        key=dummy_data.api_key, secret=dummy_data.api_secret, signature_secret="secret"
    )
    assert client.signature(params) == "6af838ef94998832dbfc29020b564830"


def test_signature_adds_timestamp(dummy_data):
    params = {"a=7": "1", "b": "2 & 5"}

    client = vonage.Client(
        key=dummy_data.api_key, secret=dummy_data.api_secret, signature_secret="secret"
    )

    client.signature(params)
    assert params["timestamp"] is not None


def test_signature_md5(dummy_data):
    params = {"a": "1", "b": "2", "timestamp": "1461605396"}
    client = vonage.Client(
        key=dummy_data.api_key,
        secret=dummy_data.api_secret,
        signature_secret=dummy_data.signature_secret,
        signature_method="md5",
    )
    assert client.signature(params) == "c15c21ced558c93a226c305f58f902f2"


def test_signature_sha1(dummy_data):
    params = {"a": "1", "b": "2", "timestamp": "1461605396"}
    client = vonage.Client(
        key=dummy_data.api_key,
        secret=dummy_data.api_secret,
        signature_secret=dummy_data.signature_secret,
        signature_method="sha1",
    )
    assert client.signature(params) == "3e19a4e6880fdc2c1426bfd0587c98b9532f0210"


def test_signature_sha256(dummy_data):
    params = {"a": "1", "b": "2", "timestamp": "1461605396"}
    client = vonage.Client(
        key=dummy_data.api_key,
        secret=dummy_data.api_secret,
        signature_secret=dummy_data.signature_secret,
        signature_method="sha256",
    )
    assert (
        client.signature(params)
        == "a321e824b9b816be7c3f28859a31749a098713d39f613c80d455bbaffae1cd24"
    )


def test_signature_sha512(dummy_data):
    params = {"a": "1", "b": "2", "timestamp": "1461605396"}
    client = vonage.Client(
        key=dummy_data.api_key,
        secret=dummy_data.api_secret,
        signature_secret=dummy_data.signature_secret,
        signature_method="sha512",
    )
    assert (
        client.signature(params)
        == "812a18f76680fa0fe1b8bd9ee1625466ceb1bd96242e4d050d2cfd9a7b40166c63ed26ec9702168781b6edcf1633db8ff95af9341701004eec3fcf9550572ee8"
    )


def test_client_doesnt_require_api_key():
    client = vonage.Client(application_id="myid", private_key="abc\nde")
    assert client is not None
    assert client.api_key is None
    assert client.api_secret is None


@responses.activate
def test_client_can_make_application_requests_without_api_key(dummy_data):
    stub(responses.POST, "https://api.nexmo.com/v1/calls")

    client = vonage.Client(application_id="myid", private_key=dummy_data.private_key)
    voice = vonage.Voice(client)
    voice.create_call("123455")


@responses.activate
def test_get_recording(client, dummy_data):
    stub_bytes(
        responses.GET,
        "https://api.nexmo.com/v1/files/d6e47a2e-3414-11e8-8c2c-2f8b643ed957",
    )

    assert isinstance(
        client.get_recording(
            "https://api.nexmo.com/v1/files/d6e47a2e-3414-11e8-8c2c-2f8b643ed957"
        ),
        bytes_type,
    )
    assert request_user_agent() == dummy_data.user_agent



from util import *


@responses.activate
def test_redact_transaction(client, dummy_data):
    responses.add(
        responses.POST,
        "https://api.nexmo.com/v1/redact/transaction",
        body=None,
        status=204,
    )

    assert client.redact_transaction(id="not-a-real-id", product="sms") is None
    assert request_user_agent() == dummy_data.user_agent
    assert request_content_type() == "application/json"


@responses.activate
def test_redact_transaction_with_type(client, dummy_data):
    responses.add(
        responses.POST,
        "https://api.nexmo.com/v1/redact/transaction",
        body=None,
        status=204,
    )

    assert client.redact_transaction(id="some-id", product="sms", type="xyz") is None
    assert request_user_agent() == dummy_data.user_agent
    assert request_content_type() == "application/json"
    assert b"xyz" in request_body()



from ._internal import ApplicationV2, BasicAuthenticatedServer, _format_date_param
from .errors import *
from .voice import *
from .sms import *
from .verify import *
from datetime import datetime
import logging
from platform import python_version

import base64
import hashlib
import hmac
import jwt
import os
import pytz
import requests
import sys
import time
from uuid import uuid4
import warnings
import re
from deprecated import deprecated


string_types = (str, bytes)
from urllib.parse import urlparse

try:
    from json import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError


__version__ = "2.4.0"

logger = logging.getLogger("nexmo")


class Client:
    """
    Create a Client object to start making calls to Nexmo APIs.

    Most methods corresponding to Nexmo API calls are on this class itself,
    although newer APIs are under namespaces like :attr:`Client.application_v2`.

    The credentials you provide when instantiating a Client determine which
    methods can be called. Consult the `Nexmo API docs <https://developer.nexmo.com/api/>`_ for details of the
    authentication used by the APIs you wish to use, and instantiate your
    Client with the appropriate credentials.

    :param str key: Your Nexmo API key
    :param str secret: Your Nexmo API secret.
    :param str signature_secret: Your Nexmo API signature secret.
        You may need to have this enabled by Nexmo support. It is only used for SMS authentication.
    :param str signature_method:
        The encryption method used for signature encryption. This must match the method
        configured in the Nexmo Dashboard. We recommend `sha256` or `sha512`.
        This should be one of `md5`, `sha1`, `sha256`, or `sha512` if using HMAC digests.
        If you want to use a simple MD5 hash, leave this as `None`.
    :param str application_id: Your application ID if calling methods which use JWT authentication.
    :param str private_key: Your private key if calling methods which use JWT authentication.
        This should either be a str containing the key in its PEM form, or a path to a private key file.
    :param str app_name: This optional value is added to the user-agent header
        provided by this library and can be used by Nexmo to track your app statistics.
    :param str app_version: This optional value is added to the user-agent header
        provided by this library and can be used by Nexmo to track your app statistics.
    """

    def __init__(
        self,
        key=None,
        secret=None,
        signature_secret=None,
        signature_method=None,
        application_id=None,
        private_key=None,
        app_name=None,
        app_version=None,
    ):
        self.api_key = key or os.environ.get("VONAGE_API_KEY", None)

        self.api_secret = secret or os.environ.get("VONAGE_API_SECRET", None)

        self.signature_secret = signature_secret or os.environ.get(
            "VONAGE_SIGNATURE_SECRET", None
        )

        self.signature_method = signature_method or os.environ.get(
            "VONAGE_SIGNATURE_METHOD", None
        )

        if self.signature_method in {"md5", "sha1", "sha256", "sha512"}:
            self.signature_method = getattr(hashlib, signature_method)

        self.application_id = application_id

        self.private_key = private_key

        if isinstance(self.private_key, string_types) and "\n" not in self.private_key:
            with open(self.private_key, "rb") as key_file:
                self.private_key = key_file.read()

        self.__host_pattern = "^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$|^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)+([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$"

        self.__host = "rest.nexmo.com"

        self.__api_host = "api.nexmo.com"

        user_agent = "vonage-python/{version} python/{python_version}".format(
            version=__version__, python_version=python_version()
        )

        if app_name and app_version:
            user_agent += " {app_name}/{app_version}".format(
                app_name=app_name, app_version=app_version
            )

        self.headers = {"User-Agent": user_agent}

        self.auth_params = {}

        api_server = BasicAuthenticatedServer(
            "https://api.nexmo.com",
            user_agent=user_agent,
            api_key=self.api_key,
            api_secret=self.api_secret,
        )
        self.application_v2 = ApplicationV2(api_server)

        self.session = requests.Session()

    # Get and Set __host attribute
    def host(self, value=None):
        if value is None:
            return self.__host
        elif not re.match(self.__host_pattern, value):
            raise Exception("Error: Invalid format for host")
        else:
            self.__host = value

    # Gets And sets __api_host attribute
    def api_host(self, value=None):
        if value is None:
            return self.__api_host
        elif not re.match(self.__host_pattern, value):
            raise Exception("Error: Invalid format for api_host")
        else:
            self.__api_host = value

    def auth(self, params=None, **kwargs):
        self.auth_params = params or kwargs

    @deprecated(
        reason="vonage.Client#send_message is deprecated. Use Sms#send_message instead"
    )
    def send_message(self, params):
        """
        Send an SMS message.
        Requires a client initialized with `key` and either `secret` or `signature_secret`.
        ::
            client.send_message({
                "to": MY_CELLPHONE,
                "from": MY_VONAGE_NUMBER,
                "text": "Hello From Nexmo!",
            })
        :param dict params: A dict of values described at `Send an SMS <https://developer.nexmo.com/api/sms#send-an-sms>`_
        """
        return self.post(self.host(), "/sms/json", params, supports_signature_auth=True)

    def get_balance(self):
        return self.get(self.host(), "/account/get-balance")

    def get_country_pricing(self, country_code):
        return self.get(
            self.host(), "/account/get-pricing/outbound", {"country": country_code}
        )

    def get_prefix_pricing(self, prefix):
        return self.get(
            self.host(), "/account/get-prefix-pricing/outbound", {"prefix": prefix}
        )

    def get_sms_pricing(self, number):
        return self.get(
            self.host(), "/account/get-phone-pricing/outbound/sms", {"phone": number}
        )

    def get_voice_pricing(self, number):
        return self.get(
            self.host(), "/account/get-phone-pricing/outbound/voice", {"phone": number}
        )

    def update_settings(self, params=None, **kwargs):
        return self.post(self.host(), "/account/settings", params or kwargs)

    def topup(self, params=None, **kwargs):
        return self.post(self.host(), "/account/top-up", params or kwargs)

    def get_account_numbers(self, params=None, **kwargs):
        return self.get(self.host(), "/account/numbers", params or kwargs)

    def get_available_numbers(self, country_code, params=None, **kwargs):
        return self.get(
            self.host(), "/number/search", dict(params or kwargs, country=country_code)
        )

    def buy_number(self, params=None, **kwargs):
        return self.post(self.host(), "/number/buy", params or kwargs)

    def cancel_number(self, params=None, **kwargs):
        return self.post(self.host(), "/number/cancel", params or kwargs)

    def update_number(self, params=None, **kwargs):
        return self.post(self.host(), "/number/update", params or kwargs)

    def get_message(self, message_id):
        return self.get(self.host(), "/search/message", {"id": message_id})

    def get_message_rejections(self, params=None, **kwargs):
        return self.get(self.host(), "/search/rejections", params or kwargs)

    def search_messages(self, params=None, **kwargs):
        return self.get(self.host(), "/search/messages", params or kwargs)

    def send_ussd_push_message(self, params=None, **kwargs):
        return self.post(self.host(), "/ussd/json", params or kwargs)

    def send_ussd_prompt_message(self, params=None, **kwargs):
        return self.post(self.host(), "/ussd-prompt/json", params or kwargs)

    def send_2fa_message(self, params=None, **kwargs):
        return self.post(self.host(), "/sc/us/2fa/json", params or kwargs)

    def submit_sms_conversion(self, message_id, delivered=True, timestamp=None):
        """
        Notify Nexmo that an SMS was successfully received.

        :param message_id: The `message-id` str returned by the send_message call.
        :param delivered: A `bool` indicating that the message was or was not successfully delivered.
        :param timestamp: A `datetime` object containing the time the SMS arrived.
        :return: The parsed response from the server. On success, the bytestring b'OK'
        """
        params = {
            "message-id": message_id,
            "delivered": delivered,
            "timestamp": timestamp or datetime.now(pytz.utc),
        }
        # Ensure timestamp is a string:
        _format_date_param(params, "timestamp")
        return self.post(self.api_host(), "/conversions/sms", params)

    def send_event_alert_message(self, params=None, **kwargs):
        return self.post(self.host(), "/sc/us/alert/json", params or kwargs)

    def send_marketing_message(self, params=None, **kwargs):
        return self.post(self.host(), "/sc/us/marketing/json", params or kwargs)

    def get_event_alert_numbers(self):
        return self.get(self.host(), "/sc/us/alert/opt-in/query/json")

    def resubscribe_event_alert_number(self, params=None, **kwargs):
        return self.post(
            self.host(), "/sc/us/alert/opt-in/manage/json", params or kwargs
        )

    def initiate_call(self, params=None, **kwargs):
        return self.post(self.host(), "/call/json", params or kwargs)

    def initiate_tts_call(self, params=None, **kwargs):
        return self.post(self.api_host(), "/tts/json", params or kwargs)

    def initiate_tts_prompt_call(self, params=None, **kwargs):
        return self.post(self.api_host(), "/tts-prompt/json", params or kwargs)

    @deprecated(
        reason="vonage.Client#start_verification is deprecated. Use Verify#start_verification instead"
    )
    def start_verification(self, params=None, **kwargs):
        return self.post(self.api_host(), "/verify/json", params or kwargs)

    def send_verification_request(self, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#send_verification_request is deprecated (use Verify#start_verification instead)",
            DeprecationWarning,
            stacklevel=2,
        )

        return self.post(self.api_host(), "/verify/json", params or kwargs)

    @deprecated(
        reason="vonage.Client#check_verification is deprecated. Use Verify#check instead"
    )
    def check_verification(self, request_id, params=None, **kwargs):
        return self.post(
            self.api_host(),
            "/verify/check/json",
            dict(params or kwargs, request_id=request_id),
        )

    def check_verification_request(self, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#check_verification_request is deprecated (use Verify#check instead)",
            DeprecationWarning,
            stacklevel=2,
        )

        return self.post(self.api_host(), "/verify/check/json", params or kwargs)

    @deprecated(
        reason="vonage.Client#start_psd2_verification_request is deprecated. Use Verify#psd2 instead"
    )
    def start_psd2_verification_request(self, params=None, **kwargs):
        return self.post(self.api_host(), "/verify/psd2/json", params or kwargs)

    @deprecated(
        reason="vonage.Client#get_verification is deprecated. Use Verify#search instead"
    )
    def get_verification(self, request_id):
        return self.get(
            self.api_host(), "/verify/search/json", {"request_id": request_id}
        )

    def get_verification_request(self, request_id):
        warnings.warn(
            "vonage.Client#get_verification_request is deprecated (use Verify#search instead)",
            DeprecationWarning,
            stacklevel=2,
        )

        return self.get(
            self.api_host(), "/verify/search/json", {"request_id": request_id}
        )

    @deprecated(
        reason="vonage.Client#cancel_verification is deprecated. Use Verify#cancel instead"
    )
    def cancel_verification(self, request_id):
        return self.post(
            self.api_host(),
            "/verify/control/json",
            {"request_id": request_id, "cmd": "cancel"},
        )

    @deprecated(
        reason="vonage.Client#trigger_next_verification_event is deprecated. Use Verify#trigger_next_event instead"
    )
    def trigger_next_verification_event(self, request_id):
        return self.post(
            self.api_host(),
            "/verify/control/json",
            {"request_id": request_id, "cmd": "trigger_next_event"},
        )

    def control_verification_request(self, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#control_verification_request is deprecated",
            DeprecationWarning,
            stacklevel=2,
        )

        return self.post(self.api_host(), "/verify/control/json", params or kwargs)

    def get_basic_number_insight(self, params=None, **kwargs):
        return self.get(self.api_host(), "/ni/basic/json", params or kwargs)

    def get_standard_number_insight(self, params=None, **kwargs):
        return self.get(self.api_host(), "/ni/standard/json", params or kwargs)

    def get_number_insight(self, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#get_number_insight is deprecated (use #get_standard_number_insight instead)",
            DeprecationWarning,
            stacklevel=2,
        )

        return self.get(self.api_host(), "/number/lookup/json", params or kwargs)

    def get_async_advanced_number_insight(self, params=None, **kwargs):
        argoparams = params or kwargs
        if "callback" in argoparams:
            return self.get(
                self.api_host(), "/ni/advanced/async/json", params or kwargs
            )
        else:
            raise ClientError(
                "Error: Callback needed for async advanced number insight"
            )

    def get_advanced_number_insight(self, params=None, **kwargs):
        return self.get(self.api_host(), "/ni/advanced/json", params or kwargs)

    def request_number_insight(self, params=None, **kwargs):
        return self.post(self.host(), "/ni/json", params or kwargs)

    def get_applications(self, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#get_applications is deprecated (use methods from #application_v2 instead)",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.get(self.api_host(), "/v1/applications", params or kwargs)

    def get_application(self, application_id):
        warnings.warn(
            "vonage.Client#get_application is deprecated (use methods from #application_v2 instead)",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.get(
            self.api_host(),
            "/v1/applications/{application_id}".format(application_id=application_id),
        )

    def create_application(self, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#create_application is deprecated (use methods from #application_v2 instead)",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.post(self.api_host(), "/v1/applications", params or kwargs)

    def update_application(self, application_id, params=None, **kwargs):
        warnings.warn(
            "vonage.Client#update_application is deprecated (use methods from #application_v2 instead)",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.put(
            self.api_host(),
            "/v1/applications/{application_id}".format(application_id=application_id),
            params or kwargs,
        )

    def delete_application(self, application_id):
        warnings.warn(
            "vonage.Client#delete_application is deprecated (use methods from #application_v2 instead)",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.delete(
            self.api_host(),
            "/v1/applications/{application_id}".format(application_id=application_id),
        )

    @deprecated(
        reason="vonage.Client#create_call is deprecated. Use Voice#create_call instead"
    )
    def create_call(self, params=None, **kwargs):
        return self._jwt_signed_post("/v1/calls", params or kwargs)

    @deprecated(
        reason="vonage.Client#get_calls is deprecated. Use Voice#get_calls instead"
    )
    def get_calls(self, params=None, **kwargs):
        return self._jwt_signed_get("/v1/calls", params or kwargs)

    @deprecated(
        reason="vonage.Client#get_call is deprecated. Use Voice#get_call instead"
    )
    def get_call(self, uuid):
        return self._jwt_signed_get("/v1/calls/{uuid}".format(uuid=uuid))

    @deprecated(
        reason="vonage.Client#update_call is deprecated. Use Voice#update_call instead"
    )
    def update_call(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}".format(uuid=uuid), params or kwargs
        )

    @deprecated(
        reason="vonage.Client#send_audio is deprecated. Use Voice#send_audio instead"
    )
    def send_audio(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}/stream".format(uuid=uuid), params or kwargs
        )

    @deprecated(
        reason="vonage.Client#stop_audio is deprecated. Use Voice#stop_audio instead"
    )
    def stop_audio(self, uuid):
        return self._jwt_signed_delete("/v1/calls/{uuid}/stream".format(uuid=uuid))

    @deprecated(
        reason="vonage.Client#send_speech is deprecated. Use Voice#send_speech instead"
    )
    def send_speech(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}/talk".format(uuid=uuid), params or kwargs
        )

    @deprecated(
        reason="vonage.Client#stop_speech is deprecated. Use Voice#stop_speech instead"
    )
    def stop_speech(self, uuid):
        return self._jwt_signed_delete("/v1/calls/{uuid}/talk".format(uuid=uuid))

    @deprecated(
        reason="vonage.Client#send_dtmf is deprecated. Use Voice#send_dtmf instead"
    )
    def send_dtmf(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}/dtmf".format(uuid=uuid), params or kwargs
        )

    def get_recording(self, url):
        hostname = urlparse(url).hostname
        return self.parse(hostname, self.session.get(url, headers=self._headers()))

    def redact_transaction(self, id, product, type=None):
        params = {"id": id, "product": product}
        if type is not None:
            params["type"] = type
        return self._post_json(self.api_host(), "/v1/redact/transaction", params)

    def list_secrets(self, api_key):
        return self.get(
            self.api_host(),
            "/accounts/{api_key}/secrets".format(api_key=api_key),
            header_auth=True,
        )

    def get_secret(self, api_key, secret_id):
        return self.get(
            self.api_host(),
            "/accounts/{api_key}/secrets/{secret_id}".format(
                api_key=api_key, secret_id=secret_id
            ),
            header_auth=True,
        )

    def create_secret(self, api_key, secret):
        body = {"secret": secret}
        return self._post_json(
            self.api_host(), "/accounts/{api_key}/secrets".format(api_key=api_key), body
        )

    def delete_secret(self, api_key, secret_id):
        return self.delete(
            self.api_host(),
            "/accounts/{api_key}/secrets/{secret_id}".format(
                api_key=api_key, secret_id=secret_id
            ),
            header_auth=True,
        )

    def check_signature(self, params):
        params = dict(params)
        signature = params.pop("sig", "").lower()
        return hmac.compare_digest(signature, self.signature(params))

    def signature(self, params):
        if self.signature_method:
            hasher = hmac.new(
                self.signature_secret.encode(), digestmod=self.signature_method
            )
        else:
            hasher = hashlib.md5()

        # Add timestamp if not already present
        if not params.get("timestamp"):
            params["timestamp"] = int(time.time())

        for key in sorted(params):
            value = params[key]

            if isinstance(value, str):
                value = value.replace("&", "_").replace("=", "_")

            hasher.update("&{key}={value}".format(key=key, value=value).encode("utf-8"))

        if self.signature_method is None:
            hasher.update(self.signature_secret.encode())

        return hasher.hexdigest()

    def get(self, host, request_uri, params=None, header_auth=False):
        uri = "https://{host}{request_uri}".format(host=host, request_uri=request_uri)
        headers = self.headers
        if header_auth:
            h = base64.b64encode(
                (
                    "{api_key}:{api_secret}".format(
                        api_key=self.api_key, api_secret=self.api_secret
                    ).encode("utf-8")
                )
            ).decode("ascii")
            headers = dict(headers or {}, Authorization="Basic {hash}".format(hash=h))
        else:
            params = dict(
                params or {}, api_key=self.api_key, api_secret=self.api_secret
            )
        logger.debug("GET to %r with params %r, headers %r", uri, params, headers)
        return self.parse(host, self.session.get(uri, params=params, headers=headers))

    def post(
        self,
        host,
        request_uri,
        params,
        supports_signature_auth=False,
        header_auth=False,
    ):
        """
        Low-level method to make a post request to a Nexmo API server.
        This method automatically adds authentication, picking the first applicable authentication method from the following:
        - If the supports_signature_auth param is True, and the client was instantiated with a signature_secret, then signature authentication will be used.
        - If the header_auth param is True, then basic authentication will be used, with the client's key and secret.
        - Otherwise the client's key and secret are appended to the post request's params.
        :param bool supports_signature_auth: Preferentially use signature authentication if a signature_secret was provided when initializing this client.
        :param bool header_auth: Use basic authentication instead of adding api_key and api_secret to the request params.
        """
        uri = "https://{host}{request_uri}".format(host=host, request_uri=request_uri)
        headers = self.headers
        if supports_signature_auth and self.signature_secret:
            params["api_key"] = self.api_key
            params["sig"] = self.signature(params)
        elif header_auth:
            h = base64.b64encode(
                (
                    "{api_key}:{api_secret}".format(
                        api_key=self.api_key, api_secret=self.api_secret
                    ).encode("utf-8")
                )
            ).decode("ascii")
            headers = dict(headers or {}, Authorization="Basic {hash}".format(hash=h))
        else:
            params = dict(params, api_key=self.api_key, api_secret=self.api_secret)
        logger.debug("POST to %r with params %r, headers %r", uri, params, headers)
        return self.parse(host, self.session.post(uri, data=params, headers=headers))

    def _post_json(self, host, request_uri, json):
        """
        Post json to `request_uri`, using basic auth.
        """
        uri = "https://{host}{request_uri}".format(host=host, request_uri=request_uri)
        auth = base64.b64encode(
            (
                "{api_key}:{api_secret}".format(
                    api_key=self.api_key, api_secret=self.api_secret
                ).encode("utf-8")
            )
        ).decode("ascii")
        headers = dict(
            self.headers or {}, Authorization="Basic {hash}".format(hash=auth)
        )
        logger.debug(
            "POST to %r with body: %r, headers: %r", request_uri, json, headers
        )
        return self.parse(host, self.session.post(uri, headers=headers, json=json))

    def put(self, host, request_uri, params, header_auth=False):
        uri = "https://{host}{request_uri}".format(host=host, request_uri=request_uri)

        headers = self.headers
        if header_auth:
            h = base64.b64encode(
                (
                    "{api_key}:{api_secret}".format(
                        api_key=self.api_key, api_secret=self.api_secret
                    ).encode("utf-8")
                )
            ).decode("ascii")
            # Must create a new headers dict here, otherwise we'd be mutating `self.headers`:
            headers = dict(headers or {}, Authorization="Basic {hash}".format(hash=h))
        else:
            params = dict(params, api_key=self.api_key, api_secret=self.api_secret)
        logger.debug("PUT to %r with params %r, headers %r", uri, params, headers)
        return self.parse(host, self.session.put(uri, json=params, headers=headers))

    def delete(self, host, request_uri, header_auth=False):
        uri = "https://{host}{request_uri}".format(host=host, request_uri=request_uri)

        params = None
        headers = self.headers
        if header_auth:
            h = base64.b64encode(
                (
                    "{api_key}:{api_secret}".format(
                        api_key=self.api_key, api_secret=self.api_secret
                    ).encode("utf-8")
                )
            ).decode("ascii")
            # Must create a new headers dict here, otherwise we'd be mutating `self.headers`:
            headers = dict(headers or {}, Authorization="Basic {hash}".format(hash=h))
        else:
            params = {"api_key": self.api_key, "api_secret": self.api_secret}
        logger.debug("DELETE to %r with params %r, headers %r", uri, params, headers)
        return self.parse(
            host, self.session.delete(uri, params=params, headers=headers)
        )

    def parse(self, host, response):
        logger.debug("Response headers %r", response.headers)
        if response.status_code == 401:
            raise AuthenticationError
        elif response.status_code == 204:
            return None
        elif 200 <= response.status_code < 300:

            # Strip off any encoding from the content-type header:
            content_mime = response.headers.get("content-type").split(";", 1)[0]
            if content_mime == "application/json":
                return response.json()
            else:
                return response.content
        elif 400 <= response.status_code < 500:
            logger.warning(
                "Client error: %s %r", response.status_code, response.content
            )
            message = "{code} response from {host}".format(
                code=response.status_code, host=host
            )

            # Test for standard error format:
            try:
                error_data = response.json()
                if (
                    "type" in error_data
                    and "title" in error_data
                    and "detail" in error_data
                ):
                    message = "{title}: {detail} ({type})".format(
                        title=error_data["title"],
                        detail=error_data["detail"],
                        type=error_data["type"],
                    )
            except JSONDecodeError:
                pass
            raise ClientError(message)
        elif 500 <= response.status_code < 600:
            logger.warning(
                "Server error: %s %r", response.status_code, response.content
            )
            message = "{code} response from {host}".format(
                code=response.status_code, host=host
            )
            raise ServerError(message)

    def _jwt_signed_get(self, request_uri, params=None):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self.api_host(), request_uri=request_uri
        )

        return self.parse(
            self.api_host(),
            self.session.get(uri, params=params or {}, headers=self._headers()),
        )

    def _jwt_signed_post(self, request_uri, params):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self.api_host(), request_uri=request_uri
        )

        return self.parse(
            self.api_host(),
            self.session.post(uri, json=params, headers=self._headers()),
        )

    def _jwt_signed_put(self, request_uri, params):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self.api_host(), request_uri=request_uri
        )

        return self.parse(
            self.api_host(), self.session.put(uri, json=params, headers=self._headers())
        )

    def _jwt_signed_delete(self, request_uri):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self.api_host(), request_uri=request_uri
        )

        return self.parse(
            self.api_host(), self.session.delete(uri, headers=self._headers())
        )

    def _headers(self):
        token = self.generate_application_jwt()
        return dict(self.headers, Authorization=b"Bearer " + token)

    def generate_application_jwt(self, when=None):
        iat = int(when if when is not None else time.time())

        payload = dict(self.auth_params)
        payload.setdefault("application_id", self.application_id)
        payload.setdefault("iat", iat)
        payload.setdefault("exp", iat + 60)
        payload.setdefault("jti", str(uuid4()))

        token = jwt.encode(payload, self.private_key, algorithm="RS256")

        # If token is string transform it to byte type
        if(type(token) is str):
            token = bytes(token, 'utf-8')

        return token



from util import *


@responses.activate
def test_get_applications(client, dummy_data):
    stub(responses.GET, "https://api.nexmo.com/v1/applications")

    with pytest.warns(DeprecationWarning) as warning_info:
        assert isinstance(client.get_applications(), dict)
    assert request_user_agent() == dummy_data.user_agent


@responses.activate
def test_get_application(client, dummy_data):
    stub(responses.GET, "https://api.nexmo.com/v1/applications/xx-xx-xx-xx")

    with pytest.warns(DeprecationWarning) as warning_info:
        assert isinstance(client.get_application("xx-xx-xx-xx"), dict)
    assert request_user_agent() == dummy_data.user_agent


@responses.activate
def test_create_application(client, dummy_data):
    stub(responses.POST, "https://api.nexmo.com/v1/applications")

    params = {"name": "Example App", "type": "voice"}

    with pytest.warns(DeprecationWarning) as warning_info:
        assert isinstance(client.create_application(params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "name=Example+App" in request_body()
    assert "type=voice" in request_body()


@responses.activate
def test_update_application(client, dummy_data):
    stub(responses.PUT, "https://api.nexmo.com/v1/applications/xx-xx-xx-xx")

    params = {"answer_url": "https://example.com/ncco"}

    with pytest.warns(DeprecationWarning) as warning_info:
        assert isinstance(client.update_application("xx-xx-xx-xx", params), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert request_content_type() == "application/json"
    assert b'"answer_url": "https://example.com/ncco"' in request_body()


@responses.activate
def test_delete_application(client, dummy_data):
    responses.add(
        responses.DELETE,
        "https://api.nexmo.com/v1/applications/xx-xx-xx-xx",
        status=204,
    )

    with pytest.warns(DeprecationWarning) as warning_info:
        assert client.delete_application("xx-xx-xx-xx") is None
    assert request_user_agent() == dummy_data.user_agent



import logging

from requests.sessions import Session

from .errors import AuthenticationError, ClientError, ServerError

try:
    from json import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError

logger = logging.getLogger("nexmo")


class BasicAuthenticatedServer(object):
    def __init__(self, host, user_agent, api_key, api_secret, timeout=None):
        self._host = host
        self._session = session = Session()
        self.timeout = None
        session.auth = (api_key, api_secret)  # Basic authentication.
        session.headers.update({"User-Agent": user_agent})

    def _uri(self, path):
        return "{host}{path}".format(host=self._host, path=path)

    def get(self, path, params=None, headers=None):
        return self._parse(
            self._session.get(self._uri(path), params=params, headers=headers, timeout=self.timeout)
        )

    def post(self, path, body=None, headers=None):
        return self._parse(
            self._session.post(self._uri(path), json=body, headers=headers, timeout=self.timeout)
        )

    def put(self, path, body=None, headers=None):
        return self._parse(
            self._session.put(self._uri(path), json=body, headers=headers, timeout=self.timeout)
        )

    def delete(self, path, body=None, headers=None):
        return self._parse(
            self._session.delete(self._uri(path), json=body, headers=headers, timeout=self.timeout)
        )

    def _parse(self, response):
        logger.debug("Response headers %r", response.headers)
        if response.status_code == 401:
            raise AuthenticationError()
        elif response.status_code == 204:
            return None
        elif 200 <= response.status_code < 300:
            return response.json()
        elif 400 <= response.status_code < 500:
            logger.warning(
                "Client error: %s %r", response.status_code, response.content
            )
            message = "{code} response".format(code=response.status_code)
            # Test for standard error format:
            try:
                error_data = response.json()
                if (
                    "type" in error_data
                    and "title" in error_data
                    and "detail" in error_data
                ):
                    message = "{title}: {detail} ({type})".format(
                        title=error_data["title"],
                        detail=error_data["detail"],
                        type=error_data["type"],
                    )
            except JSONDecodeError:
                pass
            raise ClientError(message)
        elif 500 <= response.status_code < 600:
            logger.warning(
                "Server error: %s %r", response.status_code, response.content
            )
            message = "{code} response".format(code=response.status_code)
            raise ServerError(message)


class ApplicationV2(object):
    """
    Provides Application API v2 functionality.

    Don't instantiate this class yourself, access it via :py:attr:`vonage.Client.application_v2`
    """

    def __init__(self, api_server):
        self._api_server = api_server

    def create_application(self, application_data):
        """
        Create an application using the provided `application_data`.

        :param dict application_data: A JSON-style dict describing the application to be created.

        >>> client.application_v2.create_application({ 'name': 'My Cool App!' })

        Details of the `application_data` dict are described at https://developer.nexmo.com/api/application.v2#createApplication
        """
        return self._api_server.post("/v2/applications", application_data)

    def get_application(self, application_id):
        """
        Get application details for the application with `application_id`.

        The format of the returned dict is described at https://developer.nexmo.com/api/application.v2#getApplication

        :param str application_id: The application ID.
        :rtype: dict
        """

        return self._api_server.get(
            "/v2/applications/{application_id}".format(application_id=application_id),
            headers={"content-type": "application/json"},
        )

    def update_application(self, application_id, params):
        """
        Update the application with `application_id` using the values provided in `params`.


        """
        return self._api_server.put(
            "/v2/applications/{application_id}".format(application_id=application_id),
            params,
        )

    def delete_application(self, application_id):
        """
        Delete the application with `application_id`.
        """

        self._api_server.delete(
            "/v2/applications/{application_id}".format(application_id=application_id),
            headers={"content-type": "application/json"},
        )

    def list_applications(self, page_size=None, page=None):
        """
        List all applications for your account.

        Results are paged, so each page will need to be requested to see all applications.

        :param int page_size: The number of items in the page to be returned
        :param int page: The page number of the page to be returned.
        """
        params = _filter_none_values({"page_size": page_size, "page": page})

        return self._api_server.get(
            "/v2/applications",
            params=params,
            headers={"content-type": "application/json"},
        )


def _filter_none_values(d):
    return {k: v for k, v in d.items() if v is not None}


def _format_date_param(params, key, format="%Y-%m-%d %H:%M:%S"):
    """
    Utility function to convert datetime values to strings.

    If the value is already a str, or is not in the dict, no change is made.

    :param params: A `dict` of params that may contain a `datetime` value.
    :param key: The datetime value to be converted to a `str`
    :param format: The `strftime` format to be used to format the date. The default value is '%Y-%m-%d %H:%M:%S'
    """
    if key in params:
        param = params[key]
        if hasattr(param, "strftime"):
            params[key] = param.strftime(format)



import vonage
import warnings


class Verify:
    def __init__(self, client=None, key=None, secret=None):
        try:
            self._client = client
            if self._client is None:
                self._client = vonage.Client(key=key, secret=secret)
        except Exception as e:
            print("Error: {error_message}".format(error_message=str(e)))

    def start_verification(self, params=None, **kwargs):
        return self._client.post(
            self._client.api_host(), "/verify/json", params or kwargs
        )

    def check(self, request_id, params=None, **kwargs):
        return self._client.post(
            self._client.api_host(),
            "/verify/check/json",
            dict(params or kwargs, request_id=request_id),
        )

    def search(self, request_id):
        return self._client.get(
            self._client.api_host(), "/verify/search/json", {"request_id": request_id}
        )

    def cancel(self, request_id):
        return self._client.post(
            self._client.api_host(),
            "/verify/control/json",
            {"request_id": request_id, "cmd": "cancel"},
        )

    def trigger_next_event(self, request_id):
        return self._client.post(
            self._client.api_host(),
            "/verify/control/json",
            {"request_id": request_id, "cmd": "trigger_next_event"},
        )

    def psd2(self, params=None, **kwargs):
        return self._client.post(
            self._client.api_host(), "/verify/psd2/json", params or kwargs
        )




import vonage

class Voice():
    #application_id and private_key are needed for the calling methods
    #Passing a Nexmo Client is also possible 
    def __init__(
        self,
        client=None,
        application_id=None,
        private_key=None,
    ):
        try:
            # Client is protected
            self._client = client
            if self._client is None:
                self._client = vonage.Client(application_id=application_id, private_key=private_key)
        except Exception as e:
            print('Error: {error_message}'.format(error_message=str(e)))

    # Creates a new call session
    def create_call(self, params=None, **kwargs):
        return self._jwt_signed_post("/v1/calls", params or kwargs)
    
    # Get call history paginated. Pass start and end dates to filter the retrieved information
    def get_calls(self, params=None, **kwargs):
        return self._jwt_signed_get("/v1/calls", params or kwargs)
    
    # Get a single call record by identifier
    def get_call(self, uuid):
        return self._jwt_signed_get("/v1/calls/{uuid}".format(uuid=uuid))
    
    # Update call data using custom ncco
    def update_call(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}".format(uuid=uuid), params or kwargs
        )
    
    # Plays audio streaming into call in progress - stream_url parameter is required
    def send_audio(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}/stream".format(uuid=uuid), params or kwargs
        )
    
    # Play an speech into specified call - text parameter (text to speech) is required
    def send_speech(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}/talk".format(uuid=uuid), params or kwargs
        )
    
    # plays DTMF tones into the specified call
    def send_dtmf(self, uuid, params=None, **kwargs):
        return self._jwt_signed_put(
            "/v1/calls/{uuid}/dtmf".format(uuid=uuid), params or kwargs
        )
    
    # Stops audio recently played into specified call
    def stop_audio(self, uuid):
        return self._jwt_signed_delete("/v1/calls/{uuid}/stream".format(uuid=uuid))
    
    # Stop a speech recently played into specified call
    def stop_speech(self, uuid):
        return self._jwt_signed_delete("/v1/calls/{uuid}/talk".format(uuid=uuid))
    
    # Deprecated section
    # This methods are deprecated, to use them a definition of client with key and secret parameters is mandatory
    def initiate_call(self, params=None, **kwargs):
        return self._client.post(self._client.host(), "/call/json", params or kwargs)

    def initiate_tts_call(self, params=None, **kwargs):
        return self._client.post(self._client.api_host(), "/tts/json", params or kwargs)

    def initiate_tts_prompt_call(self, params=None, **kwargs):
        return self._client.post(self._client.api_host(), "/tts-prompt/json", params or kwargs)
    # End deprecated section
    
    # Utils methods
    # _jwt_signed_post private method that Allows developer perform signed post request
    def _jwt_signed_post(self, request_uri, params):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self._client.api_host(), request_uri=request_uri
        )

        # Uses the client session to perform the call action with api
        return self._client.parse(
            self._client.api_host(), self._client.session.post(uri, json=params, headers=self._client._headers())
        )
    
    # _jwt_signed_post private method that Allows developer perform signed get request
    def _jwt_signed_get(self, request_uri, params=None):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self._client.api_host(), request_uri=request_uri
        )

        return self._client.parse(
            self._client.api_host(),
            self._client.session.get(uri, params=params or {}, headers=self._client._headers()),
        )
    
    # _jwt_signed_put private method that Allows developer perform signed put request
    def _jwt_signed_put(self, request_uri, params):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self._client.api_host(), request_uri=request_uri
        )

        return self._client.parse(
            self._client.api_host(), self._client.session.put(uri, json=params, headers=self._client._headers())
        )
    
    # _jwt_signed_put private method that Allows developer perform signed put request
    def _jwt_signed_delete(self, request_uri):
        uri = "https://{api_host}{request_uri}".format(
            api_host=self._client.api_host(), request_uri=request_uri
        )

        return self._client.parse(
            self._client.api_host(), self._client.session.delete(uri, headers=self._client._headers())
        )



import vonage, pytz
from datetime import datetime
from ._internal import _format_date_param

class Sms:
    #To init Sms class pass a client reference or a key and secret
    def __init__(
        self,
        client=None,
        key=None,
        secret=None,
        signature_secret=None,
        signature_method=None
    ):
        try:
            self._client = client
            if self._client is None:
                self._client = vonage.Client(
                    key=key,
                    secret=secret,
                    signature_secret=signature_secret,
                    signature_method=signature_method
                )
        except Exception as e:
            print('Error: {error_message}'.format(error_message=str(e)))
    
    def send_message(self, params):
        """
        Send an SMS message.
        Requires a client initialized with `key` and either `secret` or `signature_secret`.
        :param dict params: A dict of values described at `Send an SMS <https://developer.nexmo.com/api/sms#send-an-sms>`_
        """
        return self._client.post(self._client.host(), "/sms/json", params, supports_signature_auth=True)
    
    def submit_sms_conversion(self, message_id, delivered=True, timestamp=None):
        """
        Notify Nexmo that an SMS was successfully received.

        :param message_id: The `message-id` str returned by the send_message call.
        :param delivered: A `bool` indicating that the message was or was not successfully delivered.
        :param timestamp: A `datetime` object containing the time the SMS arrived.
        :return: The parsed response from the server. On success, the bytestring b'OK'
        """
        params = {
            "message-id": message_id,
            "delivered": delivered,
            "timestamp": timestamp or datetime.now(pytz.utc)
        }
        # Ensure timestamp is a string:
        _format_date_param(params, "timestamp")
        return self._client.post(self._client.api_host(), "/conversions/sms", params)



from util import *


@responses.activate
def test_get_message(client, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/search/message")

    assert isinstance(client.get_message("00A0B0C0"), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "id=00A0B0C0" in request_query()


@responses.activate
def test_get_message_rejections(client, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/search/rejections")

    assert isinstance(client.get_message_rejections(date="YYYY-MM-DD"), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "date=YYYY-MM-DD" in request_query()


@responses.activate
def test_search_messages(client, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/search/messages")

    assert isinstance(client.search_messages(to="1234567890", date="YYYY-MM-DD"), dict)
    assert request_user_agent() == dummy_data.user_agent
    assert "date=YYYY-MM-DD" in request_query()
    assert "to=1234567890" in request_query()


@responses.activate
def test_search_messages_by_ids(client, dummy_data):
    stub(responses.GET, "https://rest.nexmo.com/search/messages")

    assert isinstance(
        client.search_messages(ids=["00A0B0C0", "00A0B0C1", "00A0B0C2"]), dict
    )
    assert request_user_agent() == dummy_data.user_agent
    assert "ids=00A0B0C0" in request_query()
    assert "ids=00A0B0C1" in request_query()
    assert "ids=00A0B0C2" in request_query()
